
from grakn.client import GraknClient
import copy, re
class Store:
    
    session = None
    tx = None

    def __init__(self, host='naga.fdn', port=48555, keyspace='grakn'):
        self.HOST = host
        self.PORT = port
        self.KEYSPACE = keyspace
        self._client = None
        self.bulk_query = []

    def query(self, q):
        with self._make_client() as client:
            with client.session(keyspace=self.KEYSPACE) as session:
                with session.transaction().read() as tx:

                    #rqit is read query iterator
                    rqit = tx.query(q)
                    rs = rqit.collect_concepts()
                    for r in rs:
                        yield r
                    
    
    def _make_client(self):
        return GraknClient(uri="{}:{}".format(self.HOST, self.PORT))

    def relates(self, rel, filter=None):
        def _build_query(props,relation_name):
            Left, Right = props['relates']

            query = '''
                match 
                    $l isa {}, has {} "{}";
                    $r isa {}, has {} "{}";
                insert 
                    $R({}: $l , {}:$r) isa {};
            '''.format(Left['entity'],Left['identifier'],Left['value'],
                        Right['entity'],Right['identifier'],Right['value'],
                        Left['as'],Right['as'],relation_name)
            print(query)
            return re.sub(r'\s+',' ',query)

        relation_name = getattr(rel.__class__,'relation_name')
        props = rel.__dict__
        query = _build_query(props, relation_name)
        Left, Right = props['relates']
        if filter is None:
            self.bulk_query.append(query)
        elif filter(Left, Right):
            self.bulk_query.append(query)

        # with self._make_client() as client:
        #     with client.session(keyspace=self.KEYSPACE) as session:
        #         with session.transaction().write() as tx:
                    
        #             '''
        #                 class : relation_name
        #                 obj_struct:
        #                 {
        #                     relates:(
        #                         {
        #                             entity: Left
        #                             identifier: gid
        #                             value: <x>
        #                             as: beep
        #                         },
        #                         {
        #                             entity: Right
        #                             identifier: gid
        #                             value: <y>
        #                             as: bop
        #                         }
        #                     )
        #                 }
        #             '''
        #             query = _build_query(props, relation_name)
        #             Left, Right = props['relates']
        #             if filter is None:
        #                 insert_it = tx.query(query)
        #                 concepts = insert_it.collect_concepts()
        #                 tx.commit()
        #                 print("No filter")
        #                 print(query)
        #             elif filter(Left, Right):
        #                 insert_it = tx.query(query)
        #                 concepts = insert_it.collect_concepts()
        #                 tx.commit()
        #                 print("with filter")
        #                 print(query)
        
    def save(self, o):
        # with self._make_client() as client:
        #     with client.session(keyspace=self.KEYSPACE) as session:
        #         with session.transaction().write() as wtx:

        entity_name = getattr(o.__class__,'entity_name')
        props = o.__dict__
        fields = ', '.join([ 'has {} "{}"'.format(key, props[key]) for key in props if props[key] != ''])
        query = 'insert $x isa {}, {}; '.format(entity_name,fields)
        self.bulk_query.append(query)

        # if len(fields) > 0 :
        #     print(query)
        #     insert_it = wtx.query(query)
        #     concepts = insert_it.collect_concepts()
        #     print("inserted content with id {}".format(concepts[0].id))
        #     wtx.commit()
        # else:
        #     print("ignoring query for object : {}".format(o.__dict__))

        
    
    def flush(self):
        with self._make_client() as client:
            with client.session(keyspace=self.KEYSPACE) as session:
                with session.transaction().write() as wtx:
                    # q = " ".join(self.bulk_query)
                    # print("executing :\n< {} >".format(q))
                    for q in self.bulk_query:
                        
                        try:
                            print("writing: <{}>".format(q))
                            insert_it = wtx.query(q)
                            concepts = insert_it.collect_concepts()
                            id = concepts[0].id
                        except Exception as e:
                            Exception("Insert not returns id, it may caused by insertion failed.")
                        
                    print("commiting...")
                    wtx.commit()
                    del self.bulk_query
                    self.bulk_query = []


    def update(self, o, update_by):
        
        entity_name = getattr(o.__class__,'entity_name')
        props = o.__dict__
        crit_prop = copy.deepcopy(props)
        criteria = [(key, crit_prop[key]) for key in update_by]

        del props['name']
        del props['gid']

        fields = ', '.join([ 'has {} "{}"'.format(key, props[key]) for key in props if props[key] != ''])
        criteria = ', '.join(['has {} "{}"'.format(k, v) for k, v in criteria])
        if len(fields) > 0:
            query = 'match $x isa {}, {} ; insert $x {}; '.format(entity_name,criteria,fields)
            self.bulk_query.append(query)


import click
import os
from fdnloader.model import Brand, Product, Category, CategoryTree, CategorizedRel, ProducedRel
from fdnloader.store import Store
from fdnloader.filter import *
from fdnloader.loader import *

KEYSPACE = 'schema_check'
# HOST = '35.240.223.141'
# 35.240.223.141
HOST = 'localhost'
CURDIR = os.path.dirname(os.path.abspath(__file__))
xstore = Store(keyspace=KEYSPACE, host=HOST)

@click.command()
def load_brand_cmd(base, pattern):
    fpath = os.path.join(CURDIR, 'shapeshift/brand-all.csv')
    df = pd.read_csv(fpath)

    df = load_file_chunk(base_path=base, path_pattern=pattern)
    df = filter_brand_entity(df)

    load_brand(df)

@click.command()
def load_product_cmd(base, pattern):
    fpath = os.path.join(CURDIR, 'shapeshift/product-all.csv')
    df = pd.read_csv(fpath)
    print(df.head)

    #significant part

    df = load_file_chunk(base_path=base, path_pattern=pattern)
    df = filter_product_entity(df)

    load_product(df)

@click.command()
def load_category_cmd(base, pattern):
    # fpath = os.path.join(CURDIR, 'shapeshift/categories-fdbr-to-graph.csv')
    # df = pd.read_csv(fpath)

    df = load_file_chunk(base_path=base, path_pattern=pattern)
    df = filter_category_entity(df)

    load_category(df,xstore=xstore)

@click.command()
def relate_category_product_cmd(base, pattern):
    # fpath = os.path.join(CURDIR, 'shapeshift/product-category-relation.csv')
    # df = pd.read_csv(fpath)

    df = load_file_chunk(base_path=base, path_pattern=pattern)
    df = filter_product_category_relation(df)
    relate_category_product(df, xstore=xstore)

@click.command()
def build_category_tree_cmd(base, pattern):
    # fpath = os.path.join(CURDIR, 'shapeshift/categories-fdbr-to-graph.csv')
    # df = pd.read_csv(fpath)

    df = load_file_chunk(base_path=base, path_pattern=pattern)
    df = filter_category_tree_relation(df)
    build_category_tree(df,xstore=xstore)

@click.command()
def relate_product_brand_cmd(base, pattern):
    # fpath = os.path.join(CURDIR, 'shapeshift/product-brand-relation.csv')
    # df2 = pd.read_csv(fpath)

    df2 = load_file_chunk(base_path=base, path_pattern = pattern)
    df2 = filter_product_brand_relation(df2)
    relate_product_brand(df2, xstore=xstore)
    

@click.group()
def cli():
    pass

def main():
    cli.add_command(load_brand_cmd)
    cli.add_command(load_product_cmd)
    cli.add_command(load_category_cmd)
    cli.add_command(relate_category_product_cmd)
    cli.add_command(relate_product_brand_cmd)
    cli.add_command(build_category_tree_cmd)

    cli()


class Base:
    def from_dict(self, record):
        for k in record:
            v = record[k]
            setattr(self, k, v)

class Relation:
    def relatesObj(self, entity=None, identifier=None, value=None, plays=None):
        self.L = {
            'entity': entity,
            'identifier': identifier,
            'value': value,
            'as': plays
        }
        return self

    def withObj(self, entity=None, identifier=None, value=None, plays=None):
        self.R = {
            'entity': entity,
            'identifier': identifier,
            'value': value,
            'as': plays
        }
        self.relates = (self.L, self.R)
        return self


class Brand(Base):
    entity_name = 'brand'
    def __init__(self):
        pass

class Product(Base):
    entity_name = 'product'
    def __init__(self):
        pass

class Category(Base):
    entity_name = 'category'
    def __init__(self):
        pass

class CategoryTree(Relation):
    relation_name = 'category-tree'

class ProducedRel(Relation):
    relation_name = 'produce-rel'

class CategorizedRel(Relation):
    relation_name = 'categorized-rel'



import setuptools

setuptools.setup(
    name='fdn-kg-loader',
    version='0.1.1',
    packages=setuptools.find_packages(),
    license='Restricted License, FDN license',
    long_description=open('README.txt').read(),
    entry_points={
        'console_scripts': ['fdnloader=fdnloader:main']
    },
    install_requires=[
        'grpcio==1.16.0',
        'protobuf==3.6.1',
        'grakn',
        'grakn-client',
        'click',
        'six==1.11.0'
    ]
)
